@@ -22,13 +22,10 @@
    Authors: Pavel Cisler <pavel@eazel.com>
 */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
-#include "gnome-vfs.h"
-#include "gnome-vfs-private.h"
+#include "gnome-vfs-find-directory.h"
 
+#include "gnome-vfs-cancellable-ops.h"
 
 /**
  * gnome_vfs_find_directory:
@@ -36,8 +33,9 @@
  * @kind: kind of well known directory
  * @result: newly created URI of the directory we found
  * @create_if_needed: If directory we are looking for does not exist, try to create it
- * @create_if_needed: If we don't know where trash is yet, look for it.
- * @permissions: If creating, use these permissions
+ * @find_if_needed: If we don't know where trash is yet, look for it.
+ * @permissions: If creating, use these permissions.  This parameter is
+ *               ignored when creating the Trash directories, and 0700 is used.
  * 
  * Used to return well known directories such as Trash, Desktop, etc. from different
  * file systems.
@@ -53,7 +51,7 @@
  * and then use the @create_if_needed flag to create Trash lazily when it is
  * needed for throwing away an item on a given disk.
  * 
- * Return value: An integer representing the result of the operation
+ * Returns: An integer representing the result of the operation
  **/
 GnomeVFSResult	
 gnome_vfs_find_directory (GnomeVFSURI 			*near_uri,
