@@ -24,12 +24,32 @@
 #ifndef GNOME_VFS_HANDLE_H
 #define GNOME_VFS_HANDLE_H
 
-#include <libgnomevfs/gnome-vfs-method.h>
+#include <libgnomevfs/gnome-vfs-context.h>
+#include <libgnomevfs/gnome-vfs-file-size.h>
+#include <libgnomevfs/gnome-vfs-file-info.h>
 
 G_BEGIN_DECLS
 
+typedef gpointer GnomeVFSMethodHandle;
 typedef struct GnomeVFSHandle GnomeVFSHandle;
 
+/* Open mode.  If you don't set `GNOME_VFS_OPEN_RANDOM', you have to access the
+   file sequentially.  */
+typedef enum {
+        GNOME_VFS_OPEN_NONE = 0,
+        GNOME_VFS_OPEN_READ = 1 << 0,
+        GNOME_VFS_OPEN_WRITE = 1 << 1,
+        GNOME_VFS_OPEN_RANDOM = 1 << 2
+} GnomeVFSOpenMode;
+
+/* This is used to specify the start position for seek operations.  */
+typedef enum {
+        GNOME_VFS_SEEK_START,
+        GNOME_VFS_SEEK_CURRENT,
+        GNOME_VFS_SEEK_END
+} GnomeVFSSeekPosition;
+
+
 GnomeVFSHandle * gnome_vfs_handle_new                (GnomeVFSURI             *uri,
 						      GnomeVFSMethodHandle    *method_handle,
 						      GnomeVFSOpenMode         open_mode);
