@@ -21,17 +21,14 @@
    Author: Ettore Perazzoli <ettore@gnu.org>
 */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
+#include "gnome-vfs-iobuf.h"
 
+#include <glib/gmem.h>
+#include <glib/gmessages.h>
 #include <unistd.h>
 #include <string.h>
 
-#include "gnome-vfs.h"
-#include "gnome-vfs-private.h"
-
-
 #define BUFFER_SIZE 4096
 
 struct Buffer {
@@ -263,4 +260,3 @@ gnome_vfs_iobuf_flush (GnomeVFSIOBuf *iobuf)
 
 	return flush (iobuf);
 }
-
