@@ -31,7 +31,7 @@
 #include <glib/gstrfuncs.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-init.h>
-#include <libgnomevfs-pthread/gnome-vfs-job.h>
+#include <libgnomevfs/gnome-vfs-job.h>
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -44,7 +44,8 @@ static GnomeVFSAsyncHandle *test_handle;
 static gpointer test_callback_data;
 static gboolean test_done;
 
-#define MAX_THREAD_WAIT 100
+#define MAX_THREAD_WAIT 500
+#define MAX_FD_CHECK 128
 
 static void
 stop_after_log (const char *domain, GLogLevelFlags level, 
@@ -72,10 +73,10 @@ get_free_file_descriptor_count (void)
 {
 	int count;
 	GList *list, *p;
-	int fd;
+	int fd = -1;
 
 	list = NULL;
-	for (count = 0; ; count++) {
+	for (count = 0; fd < MAX_FD_CHECK; count++) {
 		fd = open ("/dev/null", O_RDONLY);
 		if (fd == -1) {
 			break;
@@ -109,7 +110,7 @@ wait_for_boolean (gboolean *wait_for_it)
 	}
 
 	for (i = 0; i < MAX_THREAD_WAIT; i++) {
-		usleep (1);
+		g_thread_yield ();
 		g_main_context_iteration (NULL, FALSE);
 		if (*wait_for_it) {
 			return TRUE;
@@ -128,12 +129,13 @@ wait_until_vfs_jobs_gone (void)
 	}
 
 	for (i = 0; i < MAX_THREAD_WAIT; i++) {
-		usleep (1);
+		g_thread_yield ();
 		g_main_context_iteration (NULL, FALSE);
 		if (gnome_vfs_job_get_count () == 0) {
 			return TRUE;
 		}
 	}
+	
 	return FALSE;
 }
 
@@ -147,7 +149,7 @@ wait_until_vfs_jobs_gone_no_main (void)
 	}
 
 	for (i = 0; i < MAX_THREAD_WAIT; i++) {
-		usleep (1);
+		g_thread_yield ();
 		if (gnome_vfs_job_get_count () == 0) {
 			return TRUE;
 		}
@@ -165,7 +167,7 @@ wait_until_file_descriptors_gone (void)
 	}
 
 	for (i = 0; i < MAX_THREAD_WAIT; i++) {
-		usleep (1);
+		g_thread_yield ();
 		g_main_context_iteration (NULL, FALSE);
 		if (get_used_file_descriptor_count () == 0) {
 			return TRUE;
@@ -217,6 +219,7 @@ first_get_file_info (void)
 	gnome_vfs_async_get_file_info (&test_handle,
 				       uri_list,
 				       GNOME_VFS_FILE_INFO_DEFAULT,
+				       0,
 				       get_file_info_callback,
 				       test_callback_data);
 	g_list_free (uri_list);
@@ -242,9 +245,10 @@ test_get_file_info (void)
 	gnome_vfs_async_get_file_info (&test_handle,
 				       uri_list,
 				       GNOME_VFS_FILE_INFO_DEFAULT,
+				       0,
 				       get_file_info_callback,
 				       test_callback_data);
-	g_list_free (uri_list);
+	gnome_vfs_uri_list_free (uri_list);
 
 	/* Wait until it is done. */
 	TEST_ASSERT (wait_for_boolean (&test_done), ("get_file_info 1: callback was not called"));
@@ -260,9 +264,10 @@ test_get_file_info (void)
 	gnome_vfs_async_get_file_info (&test_handle,
 				       uri_list,
 				       GNOME_VFS_FILE_INFO_DEFAULT,
+				       0,
 				       get_file_info_callback,
 				       test_callback_data);
-	g_list_free (uri_list);
+	gnome_vfs_uri_list_free (uri_list);
 	gnome_vfs_async_cancel (test_handle);
 	g_free (test_callback_data);
 
@@ -368,6 +373,7 @@ test_open_read_close (void)
 	gnome_vfs_async_open (&test_handle,
 			      "file:///etc/passwd",
 			      GNOME_VFS_OPEN_READ,
+			      0,
 			      file_open_callback,
 			      test_callback_data);
 	TEST_ASSERT (wait_for_boolean (&file_open_flag), ("open: callback was not called"));
@@ -400,6 +406,7 @@ test_open_read_cancel_close (void)
 	gnome_vfs_async_open (&test_handle,
 			      "file:///etc/passwd",
 			      GNOME_VFS_OPEN_READ,
+			      0,
 			      file_open_callback,
 			      test_callback_data);
 	TEST_ASSERT (wait_for_boolean (&file_open_flag), ("open: callback was not called"));
@@ -433,6 +440,7 @@ test_open_close (void)
 	gnome_vfs_async_open (&test_handle,
 			      "file:///etc/passwd",
 			      GNOME_VFS_OPEN_READ,
+			      0,
 			      file_open_callback,
 			      test_callback_data);
 	TEST_ASSERT (wait_for_boolean (&file_open_flag), ("open: open callback was not called"));
@@ -445,12 +453,19 @@ test_open_close (void)
 
 	TEST_ASSERT (wait_for_boolean (&file_closed_flag), ("open close: close callback was not called"));
 	
-	TEST_ASSERT (wait_until_vfs_jobs_gone (), ("open cancel 1: job never went away"));
+	TEST_ASSERT (wait_until_vfs_jobs_gone (), ("open close 1: job never went away"));
 	TEST_ASSERT (get_used_file_descriptor_count () == 0,
-		     ("open cancel 1: %d file descriptors leaked", get_used_file_descriptor_count ()));
+		     ("open close 1: %d file descriptors leaked", get_used_file_descriptor_count ()));
 	free_at_start = get_free_file_descriptor_count ();
 }
 
+static void
+empty_close_callback (GnomeVFSAsyncHandle *handle,
+		      GnomeVFSResult result,
+		      gpointer callback_data)
+{
+}
+
 static void
 test_open_cancel (void)
 {
@@ -458,6 +473,7 @@ test_open_cancel (void)
 	gnome_vfs_async_open (&test_handle,
 			      "file:///etc/passwd",
 			      GNOME_VFS_OPEN_READ,
+			      0,
 			      file_open_callback,
 			      test_callback_data);
 	gnome_vfs_async_cancel (test_handle);
@@ -472,10 +488,14 @@ test_open_cancel (void)
 	gnome_vfs_async_open (&test_handle,
 			      "file:///etc/passwd",
 			      GNOME_VFS_OPEN_READ,
+			      0,
 			      file_open_callback,
 			      test_callback_data);
 	wait_until_vfs_jobs_gone_no_main ();
 	gnome_vfs_async_cancel (test_handle);
+	if (file_open_flag) { /* too quick */
+		gnome_vfs_async_close (test_handle, empty_close_callback, NULL);
+	}
 	TEST_ASSERT (wait_until_file_descriptors_gone (),
 		     ("open cancel 2: %d file descriptors leaked", get_used_file_descriptor_count ()));
 	free_at_start = get_free_file_descriptor_count ();
@@ -502,6 +522,7 @@ test_open_fail (void)
 	gnome_vfs_async_open (&test_handle,
 			      "file:///etc/mugwump-xxx",
 			      GNOME_VFS_OPEN_READ,
+			      0,
 			      file_open_fail_callback,
 			      test_callback_data);
 	TEST_ASSERT (wait_for_boolean (&file_open_flag), ("open fail 1: callback was not called"));
@@ -515,7 +536,8 @@ static void
 my_yield (int count)
 {
 	for (; count > 0; count--) {
-		usleep (100);
+		usleep (1);
+		g_thread_yield ();
 		g_main_context_iteration (NULL, FALSE);
 	}
 }
@@ -533,6 +555,7 @@ test_load_directory_cancel (int delay_till_cancel, int chunk_count)
 		 			 | GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE
 		 			 | GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
 					chunk_count,
+					0,
 					directory_load_callback,
 					&num_entries);
 	
@@ -540,7 +563,8 @@ test_load_directory_cancel (int delay_till_cancel, int chunk_count)
 	
 	directory_load_flag = FALSE;
 	gnome_vfs_async_cancel (handle);
-	TEST_ASSERT (wait_until_vfs_jobs_gone (), ("open cancel 1: job never went away"));
+	TEST_ASSERT (wait_until_vfs_jobs_gone (), ("load directory cancel 1: job never went away delay %d",
+						   delay_till_cancel));
 	TEST_ASSERT (!directory_load_flag, ("load directory cancel 1: load callback was called"));
 
 	gnome_vfs_async_load_directory (&handle,
@@ -549,6 +573,7 @@ test_load_directory_cancel (int delay_till_cancel, int chunk_count)
 		 			 | GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE
 		 			 | GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
 					chunk_count,
+					0,
 					directory_load_callback,
 					&num_entries);
 	
@@ -556,7 +581,8 @@ test_load_directory_cancel (int delay_till_cancel, int chunk_count)
 	
 	directory_load_flag = FALSE;
 	gnome_vfs_async_cancel (handle);
-	TEST_ASSERT (wait_until_vfs_jobs_gone (), ("open cancel 2: job never went away"));
+	TEST_ASSERT (wait_until_vfs_jobs_gone (), ("load directory cancel 2: job never went away delay %d",
+						   delay_till_cancel));
 	TEST_ASSERT (!directory_load_flag, ("load directory cancel 2: load callback was called"));
 }
 
@@ -573,11 +599,12 @@ test_load_directory_fail (void)
 		 			 | GNOME_VFS_FILE_INFO_FORCE_FAST_MIME_TYPE
 		 			 | GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
 					32,
+					0,
 					directory_load_failed_callback,
 					&num_entries);
 		
-	TEST_ASSERT (wait_for_boolean (&directory_load_failed_flag), ("load directory cancel 1: load callback was not called"));
-	TEST_ASSERT (wait_until_vfs_jobs_gone (), ("open cancel 1: job never went away"));
+	TEST_ASSERT (wait_for_boolean (&directory_load_failed_flag), ("load directory 1: load callback was not called"));
+	TEST_ASSERT (wait_until_vfs_jobs_gone (), ("load directory 1: job never went away"));
 }
 
 static gboolean find_directory_flag;
@@ -616,26 +643,28 @@ test_find_directory (int delay_till_cancel)
 	find_directory_flag = FALSE;
 	
 	gnome_vfs_async_find_directory (&handle, vfs_uri_as_list,
-		GNOME_VFS_DIRECTORY_KIND_TRASH, FALSE, TRUE, 0777,
+		GNOME_VFS_DIRECTORY_KIND_TRASH, FALSE, TRUE, 0777, 0,
 		test_find_directory_callback, &find_directory_flag);
 		
-	TEST_ASSERT (wait_for_boolean (&find_directory_flag), ("find directory cancel 1: callback was not called"));
+	TEST_ASSERT (wait_for_boolean (&find_directory_flag),
+		     ("find directory cancel 1: callback was not called %d",
+		      delay_till_cancel));
 	
 	find_directory_flag = FALSE;
 	
 	gnome_vfs_async_find_directory (&handle, vfs_uri_as_list,
-		GNOME_VFS_DIRECTORY_KIND_TRASH, FALSE, TRUE, 0777,
+		GNOME_VFS_DIRECTORY_KIND_TRASH, FALSE, TRUE, 0777, 0,
 		test_find_directory_callback, &find_directory_flag);
 	
 	usleep (delay_till_cancel * 100);
 	
 	gnome_vfs_async_cancel (handle);
-	TEST_ASSERT (wait_until_vfs_jobs_gone (), ("open cancel 2: job never went away"));
+	TEST_ASSERT (wait_until_vfs_jobs_gone (), ("find directory cancel 2: job never went away"));
 	TEST_ASSERT (!find_directory_flag, ("find directory cancel 2: callback was called"));
 
 	
 	gnome_vfs_async_find_directory (&handle, vfs_uri_as_list,
-		GNOME_VFS_DIRECTORY_KIND_TRASH, FALSE, TRUE, 0777,
+		GNOME_VFS_DIRECTORY_KIND_TRASH, FALSE, TRUE, 0777, 0,
 		test_find_directory_callback, &find_directory_flag);
 	
 	my_yield (delay_till_cancel);
@@ -666,49 +695,74 @@ main (int argc, char **argv)
 	free_at_start = get_free_file_descriptor_count ();
 
 	/* Test to see that a simple async. call works without leaking or anything. */
+	fprintf (stderr, "Testing get file info...\n");
 	test_get_file_info ();
 	test_get_file_info ();
+	fprintf (stderr, "Testing open, close...\n");
 	test_open_close ();
 	test_open_close ();
+	fprintf (stderr, "Testing read, close...\n");
 	test_open_read_close ();
 	test_open_read_close ();
+	fprintf (stderr, "Testing cancellation...\n");
 	test_open_cancel ();
 	test_open_cancel ();
 
+	fprintf (stderr, "Testing failed opens...\n");
 	test_open_fail ();
 	test_open_fail ();
+	fprintf (stderr, "Testing read, cancel, closes...\n");
 	test_open_read_cancel_close ();
 	test_open_read_cancel_close ();
 
+	fprintf (stderr, "Testing directory loads");
 	test_load_directory_fail ();
 	test_load_directory_cancel (0, 1);
 	test_load_directory_cancel (1, 1);
 	test_load_directory_cancel (10, 1);
 	test_load_directory_cancel (100, 1);
+	fprintf (stderr, ".");
 	test_load_directory_cancel (0, 1);
 	test_load_directory_cancel (1, 1);
 	test_load_directory_cancel (10, 1);
 	test_load_directory_cancel (100, 1);
+	fprintf (stderr, ".");
 
 	test_load_directory_cancel (0, 32);
 	test_load_directory_cancel (1, 32);
 	test_load_directory_cancel (10, 32);
 	test_load_directory_cancel (100, 32);
+	fprintf (stderr, ".");
 	test_load_directory_cancel (0, 32);
 	test_load_directory_cancel (1, 32);
 	test_load_directory_cancel (10, 32);
 	test_load_directory_cancel (100, 32);
 
+	fprintf (stderr, "\nTesting directory finds");
 	test_find_directory (0);
 	test_find_directory (0);
+	fprintf (stderr, ".");
 	test_find_directory (1);
 	test_find_directory (1);
+	fprintf (stderr, ".");
 	test_find_directory (10);
 	test_find_directory (10);
+	fprintf (stderr, ".");
 	test_find_directory (100);
 	test_find_directory (100);
-		
+
+	fprintf (stderr, "\nTesting shutdown...\n");
 	gnome_vfs_shutdown ();
+
+	if (g_getenv ("_MEMPROF_SOCKET")) {
+		g_warning ("Waiting for memprof\n");
+		g_main_context_iteration (NULL, TRUE);
+	}
+
+	if (!at_least_one_test_failed) {
+		fprintf (stderr, "All tests passed successfully.\n");
+	}
+
 	/* Report to "make check" on whether it all worked or not. */
 	return at_least_one_test_failed ? EXIT_FAILURE : EXIT_SUCCESS;
 }
